#!/bin/perl
#
# generate_rss.pl
#
# 2005/4/16 Yuichiro Nakamura
# version 0.1

use POSIX 'strftime';
use 5.6.0;
use Jcode;
use File::Find;
use HTML::HeadParser;

# Target Site Configuration
$SITE_TITLE =jcode("EY Yuichiro Nakamura's Travel Diary")->utf8; # site title
$SITE_URL   ="http://www.yuichironakamura.com/";                         # site URL
$SITE_DESC  =jcode("EY̗sL")->utf8;                          # site description
$DEFAULT_CATEGORY =jcode("sL")->utf8;                                # default category

# Channel Configuration
$AUTHOR     ="Yuichiro Nakamura";                                # site author
$LANGUAGE   ="ja";                                               # language
$COPYRIGHT  ="Copyright 1997-2005";                              # site copyright
$GENERATOR  ="Yuichiro Nakamura's RSS Generator";                # this generator
$DOCS       ="http://blogs.law.harvard.edu/tech/rss";            # regal

# Local PC Setting
$DIRECTORY  ="/cygdrive/d/yuichiro.work.share/www/yuichiro.jp/"; # www root
$CONFIG_FILE="generate_rss.txt";                                 # config file

# For CRLF lovers
$CRLF       ="\n";

# Build date information
$DATE_RDF   =strftime "%Y-%m-%dT%H:%M:%S+00:00"     , localtime; # RSS 1.0 type
$DATE_RSS   =strftime "%a, %d %h %Y %H:%M:%S +0000" , localtime; # RSS 2.0 type

# Target files
$RDF_FILE   ="index.rdf"; # RSS 1.0
$RSS_FILE   ="index.xml"; # RSS 2.0

$RDF_BUF    ="";

# Check argument
if (scalar(@ARGV) != 1 || ($ARGV[0] ne '-c' && $ARGV[0] ne '-r' && $ARGV[0] ne '-a') )
{
    print "USAGE: generate_rss.pl -c : Generate Config file\n";
    print "       generate_rss.pl -r : Generate RSS file\n";
    print "       generate_rss.pl -a : Do all\n";
}

if ( $ARGV[0] eq '-c' || $ARGV[0] eq '-a')
{
    # Generate Config File
    print "\nGenerate Config file\n";
    if ( -e $CONFIG_FILE )
    {
        print "  \"$CONFIG_FILE\" already exists.\n";
        do {
            print "  Overwrite? [y/n] ";
            $in = <STDIN>;
            $in =~ s/[\r\n]+//g;
        } while ($in ne 'y' && $in ne 'n');
        exit(1) if ($in eq 'n');
    }
    open(CONF, "> $CONFIG_FILE")        || die "can't open $CONFIG_FILE\n\n";
    find(\&list_file, $DIRECTORY);
    close(CONF);
}

if ( $ARGV[0] eq '-r' || $ARGV[0] eq '-a')
{
    print "Generate RSS files\n";
    print "  RSS 1.0:$DIRECTORY$RDF_FILE\n";
    print "  RSS 2.0:$DIRECTORY$RSS_FILE\n";

    # Open files
    open(CONF, "< $CONFIG_FILE")        || die "can't open $CONFIG_FILE\nDo \"generate_rss.pl -c\" first.\n";
    open(RDF,  "> $DIRECTORY$RDF_FILE") || die "can't open $DIRECTORY$RDF_FILE\n";
    open(RSS,  "> $DIRECTORY$RSS_FILE") || die "can't open $DIRECTORY$RSS_FILE\n";

	#
	# Output RSS 1.0 headers
	#
	print RDF <<EOF;
<?xml version="1.0" encoding="utf-8"?>

<rdf:RDF
xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
xmlns:dc="http://purl.org/dc/elements/1.1/"
xmlns:sy="http://purl.org/rss/1.0/modules/syndication/"
xmlns:admin="http://webns.net/mvcb/"
xmlns:cc="http://web.resource.org/cc/"
xmlns="http://purl.org/rss/1.0/">
<channel rdf:about="$SITE_URL">

<title>$SITE_TITLE</title>
<link>$SITE_URL</link>
<description>$SITE_DESC</description>
<dc:language>$LANGUAGE</dc:language>
<dc:creator>$AUTHOR</dc:creator>
<dc:date>$DATE_RDF</dc:date>
<admin:generatorAgent rdf:resource="$GENERATOR" />

<items>
<rdf:Seq>
EOF

	#
	# Output RSS 2.0 headers
	#
	print RSS <<EOF;
<?xml version="1.0" encoding="utf-8"?>
<rss version="2.0">
<channel>
<title>$SITE_TITLE</title>
<link>$SITE_URL</link>
<description>$SITE_DESC</description>
<language>$LANGUAGE</language>
<copyright>$COPYRIGHT</copyright>
<lastBuildDate>$DATE_RSS</lastBuildDate>
<generator>$GENERATOR</generator>

<docs>http://blogs.law.harvard.edu/tech/rss</docs> 
EOF

    while ( <CONF> )
    {
        # I don't like chomp
        s/[\r\n]+//g;

        # skip comment
        s/^\#.*//;
        
        # html file information / convert to utf8
        # file, title, subject need to be TAB separated
        ($file, $title, $subject) = split("\t+", $_);
        $title = jcode($title)->utf8;
        $subject = jcode($subject)->utf8;

        if ( length($file)>0 && -e "$DIRECTORY$file" )
        {
            # if html file exists, read time stamp
            @file_stat=stat("$DIRECTORY$file");
            $rdf_mtime = strftime "%Y-%m-%dT%H:%M:%S+00:00", localtime( $file_stat[9] );
            $rss_mtime = strftime "%a, %d %h %Y %H:%M:%S +0000", localtime( $file_stat[9] );
            
            $description = '';
            
            #
            # Parse html header to find out <meta name="description" content="...">
            #
            if ( open (HTML,"< $found") )
            {
                $content='';
                while ( <HTML> )
                {
                    $content .= $_;
                }
                $p = HTML::HeadParser->new;
                $p->parse($content);
                $description = jcode($p->header('x-meta-description'))->utf8;
            }
            # If there is no description, use title as description
            if ( $description eq '' )
            {
                $description = $title;
            }

            # output each html info into RSS 1.0
            print RDF "<rdf:li rdf:resource=\"$SITE_URL$file\" />$CRLF";
            
            $RDF_BUF .= "<item rdf:about=\"$SITE_URL$file\">$CRLF";
            $RDF_BUF .= "<title>$title</title>$CRLF";
            $RDF_BUF .= "<link>$SITE_URL$file</link>$CRLF";
            $RDF_BUF .= "<description>$title</description>$CRLF";
            $RDF_BUF .= "<dc:subject>$description</dc:subject>$CRLF";
            $RDF_BUF .= "<dc:creator>$AUTHOR</dc:creator>$CRLF";
            $RDF_BUF .= "<dc:date>$rdf_mtime</dc:date>$CRLF";
            $RDF_BUF .= "</item>$CRLF";
            
            # output each file info into RSS 2.0
            print RSS "<item>$CRLF";
            print RSS "<title>$title</title>$CRLF";
            print RSS "<description>$description</description>$CRLF";
            print RSS "<link>$SITE_URL$file</link>$CRLF";
            print RSS "<guid>$SITE_URL$file</guid>$CRLF";
            print RSS "<category>$subject</category>$CRLF";
            print RSS "<pubDate>$rss_mtime</pubDate>$CRLF";
            print RSS "</item>$CRLF";
        }
    }

    #
    # Output RSS 1.0 footers
    #
    print RDF <<EOF;
</rdf:Seq>
</items>

</channel>
$RDF_BUF
</rdf:RDF>
EOF

    #
    # Output RSS 2.0 footers
    #
    print RSS <<EOF;
</channel>
</rss>
EOF

    # close files
    close(CONF);
    close(RDF);
    close(RSS);
    print "RSS files generated.\n";

}

sub list_file {
    my $found = $File::Find::name;
    my $content;
    my $p;
    my $title;
    my $category;

    # pick up html file
    if ( $found =~ m/.htm$/ || $found =~ m/.html/ )
    {
        if ( open (HTML,"< $found") )
        {
            # remove base directory
            $found =~ s/$DIRECTORY//;

            # Parse html header
            $content='';
            while ( <HTML> )
            {
                $content .= $_;
            }
            $p = HTML::HeadParser->new;
            $p->parse($content);
            
            # use <title>...</title> as title
            $title = jcode($p->header('Title'))->sjis;
            $title =~ s/\t/ /g;
            
            # use <meta name="keywords" content="...">  as category
            $category = jcode($p->header('x-meta-keywords'))->sjis;
            if ( $category eq '' )
            {
                $category = jcode($DEFAULT_CATEGORY)->sjis;
            }
            print CONF "$found\t$title\t$category\n";
        }
    }
}

